@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ import sys
 import tempfile
 import unittest
 
+from yapf.yapflib import errors
 from yapf.yapflib import file_resources
 from yapf.yapflib import py3compat
 
@@ -87,12 +88,12 @@ class GetCommandLineFilesTest(unittest.TestCase):
     file2 = os.path.join(tdir2, 'testfile2.py')
     _touch_files([file1, file2])
 
-    self.assertEqual(file_resources.GetCommandLineFiles([file1, file2],
-                                                        recursive=False),
-                     [file1, file2])
-    self.assertEqual(file_resources.GetCommandLineFiles([file1, file2],
-                                                        recursive=True),
-                     [file1, file2])
+    self.assertEqual(
+        file_resources.GetCommandLineFiles(
+            [file1, file2], recursive=False, exclude=None), [file1, file2])
+    self.assertEqual(
+        file_resources.GetCommandLineFiles(
+            [file1, file2], recursive=True, exclude=None), [file1, file2])
 
   def test_nonrecursive_find_in_dir(self):
     tdir1 = self._make_test_dir('test1')
@@ -101,24 +102,48 @@ class GetCommandLineFilesTest(unittest.TestCase):
     file2 = os.path.join(tdir2, 'testfile2.py')
     _touch_files([file1, file2])
 
-    self.assertEqual(file_resources.GetCommandLineFiles([tdir1],
-                                                        recursive=False),
-                     [file1])
+    self.assertRaises(
+        errors.YapfError,
+        file_resources.GetCommandLineFiles,
+        command_line_file_list=[tdir1],
+        recursive=False,
+        exclude=None)
 
   def test_recursive_find_in_dir(self):
     tdir1 = self._make_test_dir('test1')
     tdir2 = self._make_test_dir('test2/testinner/')
-    tdir3 = self._make_test_dir('test3/foo/bar/bas/kkk')
-    files = [os.path.join(tdir1, 'testfile1.py'),
-             os.path.join(tdir2, 'testfile2.py'),
-             os.path.join(tdir3, 'testfile3.py')]
+    tdir3 = self._make_test_dir('test3/foo/bar/bas/xxx')
+    files = [
+        os.path.join(tdir1, 'testfile1.py'),
+        os.path.join(tdir2, 'testfile2.py'), os.path.join(tdir3, 'testfile3.py')
+    ]
     _touch_files(files)
 
     self.assertEqual(
-        sorted(file_resources.GetCommandLineFiles([self.test_tmpdir],
-                                                  recursive=True)),
+        sorted(
+            file_resources.GetCommandLineFiles(
+                [self.test_tmpdir], recursive=True, exclude=None)),
         sorted(files))
 
+  def test_recursive_find_in_dir_with_exclude(self):
+    tdir1 = self._make_test_dir('test1')
+    tdir2 = self._make_test_dir('test2/testinner/')
+    tdir3 = self._make_test_dir('test3/foo/bar/bas/xxx')
+    files = [
+        os.path.join(tdir1, 'testfile1.py'),
+        os.path.join(tdir2, 'testfile2.py'), os.path.join(tdir3, 'testfile3.py')
+    ]
+    _touch_files(files)
+
+    self.assertEqual(
+        sorted(
+            file_resources.GetCommandLineFiles(
+                [self.test_tmpdir], recursive=True, exclude=['*test*3.py'])),
+        sorted([
+            os.path.join(tdir1, 'testfile1.py'),
+            os.path.join(tdir2, 'testfile2.py')
+        ]))
+
 
 class IsPythonFileTest(unittest.TestCase):
 
@@ -187,11 +212,10 @@ class WriteReformattedCodeTest(unittest.TestCase):
     shutil.rmtree(cls.test_tmpdir)
 
   def test_write_to_file(self):
-    s = u'foobar'
+    s = u'foobar\n'
     with tempfile.NamedTemporaryFile(dir=self.test_tmpdir) as testfile:
-      file_resources.WriteReformattedCode(testfile.name, s,
-                                          in_place=True,
-                                          encoding='utf-8')
+      file_resources.WriteReformattedCode(
+          testfile.name, s, in_place=True, encoding='utf-8')
       testfile.flush()
 
       with open(testfile.name) as f:
@@ -201,18 +225,16 @@ class WriteReformattedCodeTest(unittest.TestCase):
     s = u'foobar'
     stream = BufferedByteStream() if py3compat.PY3 else py3compat.StringIO()
     with stdout_redirector(stream):
-      file_resources.WriteReformattedCode(None, s,
-                                          in_place=False,
-                                          encoding='utf-8')
+      file_resources.WriteReformattedCode(
+          None, s, in_place=False, encoding='utf-8')
     self.assertEqual(stream.getvalue(), s)
 
   def test_write_encoded_to_stdout(self):
     s = '\ufeff# -*- coding: utf-8 -*-\nresult = "passed"\n'  # pylint: disable=anomalous-unicode-escape-in-string
     stream = BufferedByteStream() if py3compat.PY3 else py3compat.StringIO()
     with stdout_redirector(stream):
-      file_resources.WriteReformattedCode(None, s,
-                                          in_place=False,
-                                          encoding='utf-8')
+      file_resources.WriteReformattedCode(
+          None, s, in_place=False, encoding='utf-8')
     self.assertEqual(stream.getvalue(), s)
 
 
